/**
 * Copyright (c) 2024 Enzien Audio, Ltd.
 * 
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions, and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the phrase "powered by heavy",
 *    the heavy logo, and a hyperlink to https://enzienaudio.com, all in a visible
 *    form.
 * 
 *   2.1 If the Application is distributed in a store system (for example,
 *       the Apple "App Store" or "Google Play"), the phrase "powered by heavy"
 *       shall be included in the app description or the copyright text as well as
 *       the in the app itself. The heavy logo will shall be visible in the app
 *       itself as well.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 */

#ifndef _HEAVY_CONTEXT_WSTD_SMTHR_HPP_
#define _HEAVY_CONTEXT_WSTD_SMTHR_HPP_

// object includes
#include "HeavyContext.hpp"
#include "HvControlIf.h"
#include "HvControlBinop.h"
#include "HvSignalEnvelope.h"
#include "HvControlVar.h"
#include "HvControlCast.h"
#include "HvMath.h"
#include "HvSignalVar.h"
#include "HvSignalLine.h"

class Heavy_WSTD_SMTHR : public HeavyContext {

 public:
  Heavy_WSTD_SMTHR(double sampleRate, int poolKb=10, int inQueueKb=2, int outQueueKb=0);
  ~Heavy_WSTD_SMTHR();

  const char *getName() override { return "WSTD_SMTHR"; }
  int getNumInputChannels() override { return 2; }
  int getNumOutputChannels() override { return 2; }

  int process(float **inputBuffers, float **outputBuffer, int n) override;
  int processInline(float *inputBuffers, float *outputBuffer, int n) override;
  int processInlineInterleaved(float *inputBuffers, float *outputBuffer, int n) override;

  int getParameterInfo(int index, HvParameterInfo *info) override;
  struct Parameter {
    struct In {
      enum ParameterIn : hv_uint32_t {
        LMTR = 0xCB23DC60, // Lmtr
        MIX = 0x97D46AC6, // Mix
        SMTHR = 0xD88E26F9, // Smthr
      };
    };
  };

 private:
  HvTable *getTableForHash(hv_uint32_t tableHash) override;
  void scheduleMessageForReceiver(hv_uint32_t receiverHash, HvMessage *m) override;

  // static sendMessage functions
  static void cBinop_Q6jNR1JA_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_hwzbQ15n_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_1Wz1YndX_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_dpdcP8zr_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_prjo81HV_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_UfQFRIvr_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_vQ5zeNr7_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_jwcGQurU_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_p5mu4fiE_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_pRgF2I5T_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_tOxJWiFP_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cVar_YQ9tIiwB_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_K8kgvDt1_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cVar_slbWwOTY_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cVar_Mgfi0VPo_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cIf_J5kpr0ht_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cIf_5cGTlNfK_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void sEnv_1qLcdlNp_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void sEnv_GD5eZKsC_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_oqpIrJPJ_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_MQv9tCAX_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_pJKUCetU_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_fYldxeyh_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_o7ZTPGof_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_SnvM5Jqc_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_jbN4DwaX_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_IYgcCAcm_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_NgzyAcc4_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_FC1xdNLX_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_G9Lmi1kk_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_NSEOX0lv_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cVar_0vfKgRyN_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cIf_pFTXoAOj_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cIf_287NJUBl_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void sEnv_nY3OjRGI_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void sEnv_mAuGnT21_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_x169UVGW_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_yZd8NvRQ_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_JWYBASVs_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_hvGmcvKH_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_aj48y9uE_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_zg0iT5k4_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_VMuYDYk9_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_4xMRO3zF_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_T55p996T_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_GwQmNjAb_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_eoPzx1iq_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_eUB8CCRM_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cVar_Jwd9SzUe_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_LnEa3Jx3_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_05KqS2hH_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_rowVP1En_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cSend_GCUAG09U_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_kAVGbXjC_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_syOHeRzW_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_EJHbdBag_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_G3jbmlVP_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cReceive_RBeRi2rV_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cReceive_3X6V3ExG_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cReceive_5DBn35FG_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cReceive_wteQB2nu_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cReceive_JCaQfOQ6_sendMessage(HeavyContextInterface *, int, const HvMessage *);

  // objects
  SignalLine sLine_IbDDkUsY;
  SignalEnvelope sEnv_1qLcdlNp;
  SignalEnvelope sEnv_GD5eZKsC;
  SignalEnvelope sEnv_nY3OjRGI;
  SignalEnvelope sEnv_mAuGnT21;
  SignalLine sLine_LycTGtOf;
  SignalLine sLine_rx1Wij5b;
  ControlBinop cBinop_Q6jNR1JA;
  ControlBinop cBinop_hwzbQ15n;
  ControlBinop cBinop_1Wz1YndX;
  ControlBinop cBinop_dpdcP8zr;
  ControlBinop cBinop_vQ5zeNr7;
  ControlBinop cBinop_p5mu4fiE;
  ControlBinop cBinop_pRgF2I5T;
  ControlBinop cBinop_tOxJWiFP;
  ControlVar cVar_YQ9tIiwB;
  ControlVar cVar_slbWwOTY;
  SignalVarf sVarf_Vu5zPSW7;
  ControlVar cVar_Mgfi0VPo;
  ControlIf cIf_J5kpr0ht;
  ControlIf cIf_5cGTlNfK;
  ControlBinop cBinop_oqpIrJPJ;
  ControlBinop cBinop_pJKUCetU;
  ControlBinop cBinop_fYldxeyh;
  ControlBinop cBinop_jbN4DwaX;
  ControlBinop cBinop_NgzyAcc4;
  SignalVarf sVarf_WfHt2iUI;
  ControlVar cVar_0vfKgRyN;
  ControlIf cIf_pFTXoAOj;
  ControlIf cIf_287NJUBl;
  ControlBinop cBinop_x169UVGW;
  ControlBinop cBinop_JWYBASVs;
  ControlBinop cBinop_hvGmcvKH;
  ControlBinop cBinop_VMuYDYk9;
  ControlBinop cBinop_T55p996T;
  ControlVar cVar_Jwd9SzUe;
  ControlBinop cBinop_LnEa3Jx3;
  SignalVarf sVarf_WYXKwTLN;
  SignalVarf sVarf_7VrdkUzx;
  SignalVarf sVarf_c2evJCRF;
  SignalVarf sVarf_WfaxU6Py;
  ControlBinop cBinop_kAVGbXjC;
};

#endif // _HEAVY_CONTEXT_WSTD_SMTHR_HPP_
